CON
        _clkmode = xtal1 + pll16x                                               'Standard clock mode * crystal frequency = 80 MHz
        _xinfreq = 5_000_000

'' Indices for R/C receiver inputs 
  IN_THRO  = 0
  IN_AILE  = 1
  IN_ELEV  = 2
  IN_RUDD  = 3
  IN_AUX1  = 4
  IN_AUX2  = 5

  FL_MOTOR = 0
  FR_MOTOR = 1          
  BR_MOTOR = 2
  BL_MOTOR = 3
  OUT_AUX1 = 4
  OUT_AUX2 = 5

VAR
  long Cog                                              ' Cog count.
  long Debug[40]                                        ' Array for transmitting to cpu
  long Stack[30]                                        ' Used to fire up COGS.
  long Output[8]                                        ' Used to store values that are written to ESC

  long Armed                                            ' Used to enable motors and PID loops(Flight)
  long LastTime                                         ' Used to time sample rate
  
  long ax0,ay0                                          ' ACC zero offset values

  long GPCT,APCT                                        ' Complimentary Filter Percentages

  long Gx,Gy,Gz                                         ' Raw readings from the Gyro, they have been zeroed in the "ITG 3200 Sport pasm" OBJ.
  long GRoll,GPitch,GYaw                                ' Angle determined from the GYRO

  long Ax,Ay,Az                                         ' ACC unzeroed angles from "ITG 3200 Sport pasm" OBJ.
  long ARoll,APitch                                     ' ACC output angles              

  long Roll,Pitch,Yaw                                   ' Output angles from complimentary filter.

  long ARollLFS,APitchLFS
  long AlphaACC
  
  long Thro,Aile,Elev,Rudd,AUX1,AUX2                    ' RC input variables

  long DesiredRoll,DesiredPitch,DesiredYaw              ' Desired angles determined by input from the RC

  long RollTrim,PitchTrim,YawTrim

  long PitchA,RollA,YawA
  '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
''''''PID Variables''''''
  long DoInt
  long PIDMaxIntegral
  long PIDMaxOutput
''''''Roll PID Variables''''''
  long KpRoll,KiRoll,KdRoll
  long PErrorRoll,LstPErrorRoll,DErrorRoll,IErrorRoll
  long PIDRollOut
''''''Pitch PID Variables''''''
  long KpPitch,KiPitch,KdPitch
  long PErrorPitch,LstPErrorPitch,DErrorPitch,IErrorPitch
  long PIDPitchOut
''''''Yaw PID Variables''''''
  long KpYaw,KiYaw,KdYaw 
  long PErrorYaw,LstPErrorYaw,DErrorYaw,IErrorYaw
  long PIDYawOut
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  long DbgCountHigh,DbgCountLow,DbgCountDifference      ' Used for timeing code speed

OBJ

  RCIN     : "RC Reciever 6 Inputs"                     'Reads R/C PWM input signals.                   Requires 1 Cog. 
  Debugg   : "Extended_FDSerial"                        'Talks to PC takes                              Requires 2 Cog.
  GyroAcc  : "ITG 3200 Sport pasm"
  ESC      : "Servo 8 Outputs"                          'Sends PWM signal to ESC's.                     Requires 1 Cog.
  
PUB Main

  Initiate_Debugger_Cog
  Initiate_Variables
  Initiate_PID_Values
  Initiate_6DOF
  Initiate_RCIN
  Initiate_ESC
  'Zero_ACC
  Flight

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  
PUB Flight

  LastTime := cnt
  Repeat
    DbgCountLow  := cnt                                 'Used to time Flight loop.
    Check_Armed_Status
    Get_RC
    Get_GYRO
    Convert_GYRO_2_Angle
    Convert_ACC_2_Angle
    LowPass_Filter_ACC
    Complimentary_Filter
    Change_RC_2_Angle
    If ARMED == TRUE
      Roll_PID
      Pitch_PID
      Yaw_PID
      Motor_Output_Calc
      Write_to_ESC
    Else
      Check_Gestures
    DbgCountHigh  := cnt                                'Used to time Flight loop.   
    Debug[34] := DbgCountLow  
    Debug[35] := DbgCountHigh

    
    waitcnt( constant(80_000_000 / 250) + LastTime )
    LastTime += constant(80_000_000 / 250)
    
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>    
PUB Initiate_Debugger_Cog : Status

  Status := Cog := cognew(Debugger, @Stack[0])'Launch new cog

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  
PUB Initiate_Variables

  Dira[0]    := 1                                       'Armed LED pin.
  Outa[0]    := 0                                       'Armed LED off.
  Armed      := False                                   'Armed status False

  'ax0        := 1658'2550'2547'1026'991                                     'Zero Value for ACC X
  ay0        := -2480'-4270'-4381'-4820'-4832                                   'Zero Value for ACC Y

  GPCT       := 1000'998                                    'Sets Complimentary Filter Gyro Percentage.
  APCT       := 0'2                                      'Sets Complimentary Filter ACC Percentage.

  RollTrim   := 7
  PitchTrim  := -17
  'YawTrim    := 0

  AlphaACC   := 5
  
  Debug[13]  := ax0                                     'Loads ax0 into debug variable.
  Debug[14]  := ay0                                     'Loads ay0 into debug variable.
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Initiate_PID_Values

  DoInt          := False
  PIDMaxOutput   := 50
  PIDMaxIntegral := 2000                                '(PIDMaxIntergral * Ki) / PIDOutFactor = How much will be added to the PIDOutTotal.
  
  KpRoll  := 5
  KiRoll  := 0
  KdRoll  := 0

  KpPitch := 5
  KiPitch := 0
  KdPitch := 0

  KpYaw   := 80
  KiYaw   := 0
  KdYaw   := 0

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Initiate_6DOF

  GyroAcc.Start(6,7)

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Initiate_RCIN

  RCIN.Start
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Initiate_ESC 

  ESC.Start
  Output[0] := Output[1] := Output[2] := Output[3] := 960
  Output[4] := Output[5] := Output[6] := Output[7] := 960
  Write_to_ESC

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Zero_ACC | xc, yc, C

  xc  := 0
  yc  := 0 
  Outa[0] := 1
  C := 0
  repeat 256
    xc  := GyroAcc.GetARX
    yc  := GyroAcc.GetARY
    
    ax0 += xc 
    ay0 += yc
    waitcnt( constant(80_000_000/130) + cnt )           '130 give a 2 second time to collect readings.

    If (Outa[0] == 0) and (C == 10)                     'This section is just to flash an LED during Calibration.
      outa[0] := 1                                      '  "
      C := 0                                            '  "
    else                                                '  "
      outa[0] := 0                                      '  "
      C += 1                                            '  "

  
  ax0 := ax0 / 256
  ax0 := ax0 * 5493         
  ax0 := ax0 / 1000         
  If ax0 > 180000          
    ax0 := ax0 - 360000
  'ax0 := ax0 * -1

  ay0 := ay0 / 256
  ay0 := ay0 * 5493         
  ay0 := ay0 / 1000         
  If ay0 > 180000          
    ay0 := ay0 - 360000
  ay0 := ay0 * -1

  Debug[13]  := ax0
  Debug[14]  := ay0
  outa[0] := 0                                          ' Turns LED off because Quad is unarmed.
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Zero_Gyro
'>>>>>>>>>>> This Section Takes uS
  GyroAcc.FindZero
  GRoll  := 0
  GPitch := 0
  GYaw   := 0
  Roll   := 0
  Yaw    := 0
  Pitch  := 0
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Check_Armed_Status
'>>>>>>>>>>> This Section Takes uS

  'Left stick positioned in lower Right? Armed
    if( Thro < 1000  AND  Rudd >  400 )
      Armed   := TRUE
      Outa[0] := 1
      
  'Left stick positioned in lower Left?  UnArmed
    if( Thro < 1000  AND  Rudd < -400 )
      Armed   := FALSE
      Outa[0] := 0

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Get_RC
'>>>>>>>>>>> This Section Takes uS

  Thro := RCIN.GetRC(IN_THRO)
  Aile := RCIN.GetRC(IN_AILE)
  Elev := RCIN.GetRC(IN_ELEV)
  Rudd := RCIN.GetRC(IN_RUDD)
  AUX1 := RCIN.GetRC(IN_AUX1)
  AUX2 := RCIN.GetRC(IN_AUX2)

  Thro := Thro + 1500

  Debug[0]  := Thro
  Debug[1]  := Aile
  Debug[2]  := Elev
  Debug[3]  := Rudd
  Debug[4]  := AUX1
  Debug[5]  := AUX2

  if( ||Aile < 12 )
      Aile := 0
  if( ||Elev < 12 )
      Elev := 0
  if( ||Rudd < 12 )
      Rudd := 0
      
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Get_GYRO
'>>>>>>>>>>> This Section Takes uS
  
  Gx := GyroAcc.GetRY          
  Gy := GyroAcc.GetRX                                                                
  Gz := GyroAcc.GetRZ

  Debug[10] := Gx
  Debug[11] := Gy
  Debug[12] := Gz
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Convert_GYRO_2_Angle
'>>>>>>>>>>> This Section Takes uS
  
  Gx     := Gx * 4
  Gx     := GX * 1000
  Gx     := GX / 14375
  GRoll  := Roll + Gx

  Gy     := Gy * 4
  Gy     := Gy * 1000
  Gy     := Gy / 14375
  GPitch := Pitch + Gy

  Gz     := Gz * 4
  Gz     := Gz * 1000
  Gz     := Gz / 14375
  GYaw   := GYaw + Gz

  If DesiredYaw <> 0                                    'Used to reset gyro z to zero and to forced PID loop to output value to spin quad.
    GYaw := 0

  Debug[21] := GYaw
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Convert_ACC_2_Angle
'>>>>>>>>>>> This Section Takes uS
  
  Ax     := GyroAcc.getArx    'Read ACC Angle X from Cordic Table in ITG 3200 Sport OBJ
  Ax     := Ax * 5493         'ITG OBJ puts out value 0 to 65535 which is 0 to 360 degrees.  360/65535 =.00549324
  ARoll  := Ax / 1000         'I want 3 decimal places of resolution.
  If ARoll > 180000           'I do not want 0 to 360 I want 0 to 180 and 0 to -180.
    ARoll := ARoll - 360000
  ARoll *= -1
  ARoll := ARoll + Ax0
   
  Ay     := GyroAcc.getAry    'Read ACC Angle X from Cordic Table in ITG 3200 Sport OBJ
  Ay     := Ay * 5493         'ITG OBJ puts out value 0 to 65535 which is 0 to 360 degrees.  360/65535 =.00549324
  APitch := Ay / 1000         'I want 3 decimal places of resolution.
  If APitch > 180000          'I do not want 0 to 360 I want 0 to 180 and 0 to -180.
    APitch := APitch - 360000
  APitch := APitch + ay0

  'Debug[19] := ARoll
  'Debug[20] := APitch
  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB LowPass_Filter_ACC
'>>>>>>>>>>> This Section Takes uS

  ARoll     := ARollLFS + ((AlphaACC * (ARoll - ARollLFS))/1000)
  ARollLFS  := ARoll

  APitch    := APitchLFS + ((AlphaACC * (APitch - APitchLFS))/1000)
  APitchLFS := APitch
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Complimentary_Filter
'>>>>>>>>>>> This Section Takes uS
'>>>>>>>>>>> This section is the first complimentary filter that I have tried the section below will be a different one.
'>>>>>>>>>>> Roll = (GRoll * GYROPCT) + (ARoll * ACCPCT)

  Roll  := ((GRoll * GPCT)+(ARoll * APCT))/ 1000        'was 950 and 50

  Pitch := ((GPitch * GPCT)+(APitch * APCT))/ 1000       'was 950 and 50
  
  Debug[19] := Roll
  Debug[20] := Pitch 

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Change_RC_2_Angle
'>>>>>>>>>>> This Section Takes uS




  DesiredRoll  := Aile * 30
  DesiredPitch := Elev * 30
  DesiredYaw   := Rudd * 22

  Debug[16] := DesiredRoll
  Debug[17] := DesiredPitch
  Debug[18] := DesiredYaw

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Roll_PID
'>>>>>>>>>>> This Section Takes uS

  'DbgCountLow  := cnt
  PErrorRoll     := DesiredRoll - Roll
  DErrorRoll     := PErrorRoll - LstPErrorRoll
  LstPErrorRoll  := PErrorRoll
  PIDRollOut     := (KpRoll * PErrorRoll) + (KiRoll * IErrorRoll) +  (KdRoll * DErrorRoll)
  PIDRollOut     := PIDRollOut/10000
  PIDRollOut     := -PIDMaxOutput #> PIDRollOut <# PIDMaxOutput
    if(DoInt)
      IErrorRoll += PErrorRoll
      IErrorRoll := -PIDMaxIntegral #> IErrorRoll <# PIDMaxIntegral 
  Debug[28]      := PIDRollOut
  'DbgCountHigh := cnt

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>   
PUB Pitch_PID
'>>>>>>>>>>> This Section Takes uS

  'DbgCountLow    := cnt
  PErrorPitch     := DesiredPitch - Pitch
  DErrorPitch     := PErrorPitch - LstPErrorPitch
  LstPErrorPitch  := PErrorPitch
  PIDPitchOut     := (KpPitch * PErrorPitch) + (KiPitch * IErrorPitch) +  (KdPitch * DErrorPitch)
  PIDPitchOut     := PIDPitchOut/10000
  PIDPitchOut     := -PIDMaxOutput #> PIDPitchOut <# PIDMaxOutput
    if(DoInt)
      IErrorPitch += PErrorPitch
      IErrorPitch := -PIDMaxIntegral #> IErrorPitch <# PIDMaxIntegral 
  Debug[29]       := PIDPitchOut
  'DbgCountHigh   := cnt

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  
PUB Yaw_PID
'>>>>>>>>>>> This Section Takes uS

  'DbgCountLow    := cnt
  PErrorYaw     := DesiredYaw - Yaw
  DErrorYaw     := PErrorYaw - LstPErrorYaw
  LstPErrorYaw  := PErrorYaw
  PIDYawOut     := (KpYaw * PErrorYaw) + (KiYaw * IErrorYaw) +  (KdYaw * DErrorYaw)
  PIDYawOut     := PIDYawOut/10000
  PIDYawOut     := -PIDMaxOutput #> PIDYawOut <# PIDMaxOutput
    if(DoInt)
      IErrorYaw += PErrorYaw
      IErrorYaw := -PIDMaxIntegral #> IErrorYaw <# PIDMaxIntegral 
  Debug[30]     := PIDYawOut
  'DbgCountHigh   := cnt

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Motor_Output_Calc
'>>>>>>>>>>> This Section Takes uS
  
  If Thro > 1100
    'PIDRollOut  += RollTrim
    'PIDPitchOut += PitchTrim
    'PIDYawOut   += YawTrim
    Output[FL_MOTOR] := Thro + (-PIDPitchOut - PIDRollOut - PIDYawOut) + PitchTrim + RollTrim                         
    Output[FR_MOTOR] := Thro + (-PIDPitchOut + PIDRollOut + PIDYawOut) + PitchTrim - RollTrim
    Output[BL_MOTOR] := Thro + ( PIDPitchOut - PIDRollOut + PIDYawOut) - PitchTrim + RollTrim
    Output[BR_MOTOR] := Thro + ( PIDPitchOut + PIDRollOut - PIDYawOut) - PitchTrim - RollTrim
  Else
    Output[0] := Output[1] := Output[2] := Output[3] := 960
    Output[4] := Output[5] := Output[6] := Output[7] := 960  
'This clamps the outputs at 960uS Minimum or 2000uS Maximum.
  Output[FL_MOTOR] := 960 #> Output[FL_MOTOR] <# 2000
  Output[FR_MOTOR] := 960 #> Output[FR_MOTOR] <# 2000
  Output[BR_MOTOR] := 960 #> Output[BR_MOTOR] <# 2000
  Output[BL_MOTOR] := 960 #> Output[BL_MOTOR] <# 2000

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>  
PUB Write_to_ESC
'>>>>>>>>>>> This Section Takes uS

  ESC.Set(FL_MOTOR,Output[0])
  ESC.Set(FR_MOTOR,Output[1])
  ESC.Set(BR_MOTOR,Output[2])
  ESC.Set(BL_MOTOR,Output[3])

  Debug[6]  := Output[0]
  Debug[7]  := Output[1]
  Debug[8]  := Output[2]
  Debug[9]  := Output[3]

'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
PUB Check_Gestures
  If ( Thro > 1650 AND Aile < -350 AND Elev > 250 ) 
    Zero_ACC
    LastTime := cnt
  If ( Thro > 1650 AND Aile > 350 AND Elev > 250 )
    Zero_Gyro
    LastTime := cnt  
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 
PUB Debugger  | x, idx
         
  Debugg.start(2,1,0,115200)     '31,30  for ftdi chip
  Debugg.rxflush
  Dira[3] := 1
  Outa[3] := 0
  repeat    
    x:=Debugg.rxtime(10)       
    if x=="a"
      repeat   idx from 0 to 159
        Outa[3] := 1
        Debugg.tx(byte[@Debug+idx])
        Outa[3] := 0
      x:=-1    